#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAX_SIZE 100

typedef struct
{
    char *name;
    int lastGameScore;
    int score;
    int wins;
} Player;

/**
 * Allocates memory for a line of characters.
 *
 * @return A pointer to the allocated memory.
 */
char *AllocateLine()
{
    char *line = malloc(MAX_SIZE * sizeof(char));
    if (line == NULL)
    {
        printf("Failed to allocate memory\n");
        exit(1);
    }
    return line;
}

/**
 * @brief Creates a player based on the given line of input.
 * 
 * This function takes a line of input and creates a player object based on the information in the line.
 * The created player is added to the players array and the number of players is updated accordingly.
 * 
 * @param line The line of input containing the player information.
 * @param players The array of players.
 * @param NumberOfPlayers A pointer to the number of players.
 */
void CreatePlayer(char *line, Player **players, int *NumberOfPlayers)
{
    // alocar memória para ponteiros
    *players = realloc(*players, (*NumberOfPlayers + 1) * sizeof(Player));
    if (*players == NULL)
    {
        printf("Failed to allocate memory\n");
        exit(1);
    }

    (*players)[*NumberOfPlayers].name = malloc(strlen(line) + 1);
    if ((*players)[*NumberOfPlayers].name == NULL)
    {
        printf("Failed to allocate memory\n");
        exit(1);
    }

    strcpy((*players)[*NumberOfPlayers].name, line);
    (*players)[*NumberOfPlayers].lastGameScore = 0;
    (*players)[*NumberOfPlayers].score = 0;
    (*players)[*NumberOfPlayers].wins = 0;

    (*NumberOfPlayers)++;
}

/**
 * @brief Retrieves the name from the given line.
 * 
 * This function takes a line as input and returns the name extracted from it.
 * The name is a string of characters.
 * 
 * @param line The input line from which to extract the name.
 * @return char* The extracted name from the line.
 */
char *GetName(char *line)
{
    char *name = strtok(line, ":");
    return name;
}

/**
 * Retrieves the points.
 *
 * @return The points.
 */
int GetPoints()
{
    // strtok altera a string original, aqui line já não tem o nome.
    char *pointsStr = strtok(NULL, ":");
    int points = atoi(pointsStr);
    return points;
}

/**
 * Handles a player based on the given line of input.
 *
 * @param line The line of input containing player information.
 * @param players An array of Player structures.
 * @param NumberOfPlayers A pointer to the number of players.
 */
void HandlePlayer(char *line, Player *players, int *NumberOfPlayers)
{
    char *name = GetName(line);
    int points = GetPoints(line);

    for (int i = 0; i < *NumberOfPlayers; i++)
    {
        if (strcmp(name, players[i].name) == 0)
        {
            players[i].lastGameScore = points;
            players[i].score += points;
        }
    }
}

/**
 * Handles an empty line in the input file.
 *
 * @param readingPlayers A pointer to the flag indicating if we are currently reading players.
 * @param lastLineIsWhite A pointer to the flag indicating if the last line was empty.
 */
void HandleEmptyLine(int *readingPlayers, int *lastLineIsWhite)
{
    if (!*readingPlayers && !*lastLineIsWhite)
    {
        *lastLineIsWhite = 1;
    }

    if (*readingPlayers)
    {
        *readingPlayers = 0;
    }
}

/**
 * Updates the player information based on the given line.
 *
 * @param line The line containing the player information.
 * @param readingPlayers Pointer to the flag indicating if players are being read.
 * @param lastLineIsWhite Pointer to the flag indicating if the last line was white.
 * @param players Array of players.
 * @param NumberOfPlayers Pointer to the number of players.
 */
void UpdatePlayer(char *line, int *readingPlayers, int *lastLineIsWhite, Player *players, int *NumberOfPlayers)
{
    char *str = malloc(strlen(line));
    int num;

    if (!*readingPlayers)
    {
        if (sscanf(line, "%[^:]:%d", str, &num) == 2)
        {
            HandlePlayer(line, players, NumberOfPlayers);
        }

        *lastLineIsWhite = 0;
    }

    free(str);
}

/**
 * @brief Frees the memory allocated for an array of Player pointers.
 * 
 * This function frees the memory allocated for an array of Player pointers.
 * It takes as input the address of the array of Player pointers and the number of players.
 * 
 * @param players The address of the array of Player pointers.
 * @param NumberOfPlayers The number of players.
 */
void FreePlayers(Player **players, int NumberOfPlayers)
{
    for (int i = 0; i < NumberOfPlayers; i++)
    {
        free((*players)[i].name);
    }
    free(*players);
    *players = NULL;
}

/**
 * @brief Prints the result of the players.
 *
 * This function takes an array of Player structures and the number of players as input.
 * It prints the result of each player.
 *
 * @param players The array of Player structures.
 * @param NumberOfPlayers The number of players.
 */
void PrintResult(Player *players, int NumberOfPlayers)
{
    for (int i = 0; i < NumberOfPlayers; i++)
        printf("%s - %d pontos, %d vitorias.\n", players[i].name, players[i].score, players[i].wins);
}

/**
 * @brief Updates the winners based on the player scores.
 * 
 * This function takes an array of Player structures, the number of players, and the count of lines as input.
 * It updates the winners based on the player scores.
 * 
 * @param players The array of Player structures.
 * @param NumberOfPlayers The number of players.
 */
void UpdateWinners(Player *players, int NumberOfPlayers)
{
    int biggest = 0;
    for (int i = 0; i < NumberOfPlayers; i++)
    {
        if (players[i].lastGameScore > biggest)
        {
            biggest = players[i].lastGameScore;
        }
    }

    for (int i = 0; i < NumberOfPlayers; i++)
    {
        if (players[i].lastGameScore == biggest && biggest >= 2)
        {
            players[i].wins++;
        }
        players[i].lastGameScore = 0;
    }
}

int main()
{
    int readingPlayers = 1, lastLineIsWhite = 0, NumberOfPlayers = 0;
    char *line = AllocateLine();
    Player *players = NULL;
    
    while (fgets(line, MAX_SIZE, stdin) != NULL)
    {

        line[strcspn(line, "\n")] = 0;

        if (line[0] == '\0')
        {

            if (lastLineIsWhite && !readingPlayers)
                break;

            HandleEmptyLine(&readingPlayers, &lastLineIsWhite);

            if (!readingPlayers)
                UpdateWinners(players, NumberOfPlayers);
        }
        else
        {
            if (readingPlayers)
            {
                CreatePlayer(line, &players, &NumberOfPlayers);
            }
            else
            {
                UpdatePlayer(line, &readingPlayers, &lastLineIsWhite, players, &NumberOfPlayers);
            }
        }   
    }

    PrintResult(players, NumberOfPlayers);

    // libertar memória alocada dinamicamente
    free(line);
    FreePlayers(&players, NumberOfPlayers);
    return 0;
}
